@@ -1,5 +1,5 @@
 #! perl -w
-# $Revision: 1.104 $
+# $Revision: 1.104.2.1 $
 
 # Copyright (c) 2000-2002 Ned Konz. All rights reserved.  This program is free
 # software; you can redistribute it and/or modify it under the same terms as
@@ -41,7 +41,7 @@ BEGIN
 {
 	require Exporter;
 
-	$VERSION = "1.13";
+	$VERSION = "1.15";
 	@ISA = qw( Exporter );
 
 	my @ConstantNames = qw( FA_MSDOS FA_UNIX GPBF_ENCRYPTED_MASK
@@ -306,7 +306,7 @@ sub _binmode    # Archive::Zip
 
 # Attempt to guess whether file handle is seekable.
 # Because of problems with Windoze, this only returns true when
-# the file handle is a real file.
+# the file handle is a real file.  
 sub _isSeekable    # Archive::Zip
 {
 	my $fh = shift;
@@ -317,7 +317,7 @@ sub _isSeekable    # Archive::Zip
 	}
 	elsif ( UNIVERSAL::isa( $fh, 'IO::String' ) )
 	{
-		return 1;
+		return 1;	
 	}
 	elsif ( UNIVERSAL::can( $fh, 'stat' ) )
 	{
@@ -453,7 +453,8 @@ sub _asZipDirName    # Archive::Zip
 	my @dirs = map { $_ =~ s{/}{_}g; $_ } File::Spec->splitdir($directories);
 	if ( @dirs > 0 ) { pop (@dirs) unless $dirs[-1] }   # remove empty component
 	push ( @dirs, $file || '' );
-	return wantarray ? @dirs : join ( '/', @dirs );
+	#return wantarray ? @dirs : join ( '/', @dirs );
+    return join ( '/', @dirs );
 }
 
 # Return an absolute local name for a zip name.
@@ -638,6 +639,7 @@ sub extractMember    # Archive::Zip::Archive
 	my $member = shift;
 	$member = $self->memberNamed($member) unless ref($member);
 	return _error('member not found') unless $member;
+    my $originalSize = $member->compressedSize();
 	my $name = shift;    # local FS name if given
 	my ( $volumeName, $dirName, $fileName );
 	if ( defined($name) )
@@ -657,7 +659,10 @@ sub extractMember    # Archive::Zip::Archive
 		mkpath($dirName);
 		return _ioError("can't create dir $dirName") if ( !-d $dirName );
 	}
-	return $member->extractToFileNamed( $name, @_ );
+	my $rc = $member->extractToFileNamed( $name, @_ );
+    # TODO refactor this fix into extractToFileNamed()
+    $member->{'compressedSize'} = $originalSize;
+    return $rc; 
 }
 
 sub extractMemberWithoutPaths    # Archive::Zip::Archive
@@ -666,6 +671,7 @@ sub extractMemberWithoutPaths    # Archive::Zip::Archive
 	my $member = shift;
 	$member = $self->memberNamed($member) unless ref($member);
 	return _error('member not found') unless $member;
+    my $originalSize = $member->compressedSize();
 	return AZ_OK if $member->isDirectory();
 	my $name = shift;
 	unless ($name)
@@ -674,7 +680,9 @@ sub extractMemberWithoutPaths    # Archive::Zip::Archive
 		$name =~ s{.*/}{};    # strip off directories, if any
 		$name = Archive::Zip::_asLocalName($name);
 	}
-	return $member->extractToFileNamed( $name, @_ );
+	my $rc = $member->extractToFileNamed( $name, @_ );
+    $member->{'compressedSize'} = $originalSize;
+    return $rc;
 }
 
 sub addMember    # Archive::Zip::Archive
@@ -766,6 +774,9 @@ sub writeToFileHandle    # Archive::Zip::Archive
 {
 	my $self         = shift;
 	my $fh           = shift;
+    return _error('No filehandle given') unless $fh;
+    return _ioError('filehandle not open') unless $fh->opened();
+
 	my $fhIsSeekable = @_ ? shift: _isSeekable($fh);
 	_binmode($fh);
 
@@ -2437,7 +2448,7 @@ sub fileName    # Archive::Zip::DirectoryMember
 # is in their code...
 sub contents
 {
-	undef;
+	 return wantarray ? ( undef, AZ_OK ) : undef;
 }
 
 # ----------------------------------------------------------------------
@@ -2929,14 +2940,16 @@ sub _readCentralDirectoryFileHeader    # Archive::Zip::ZipFileMember
 		return _ioError("reading central dir header");
 	}
 	my ( $fileNameLength, $extraFieldLength, $fileCommentLength );
-	( $self->{'versionMadeBy'},          $self->{'fileAttributeFormat'},
-	  $self->{'versionNeededToExtract'}, $self->{'bitFlag'},
-	  $self->{'compressionMethod'},      $self->{'lastModFileDateTime'},
-	  $self->{'crc32'},                  $self->{'compressedSize'},
-	  $self->{'uncompressedSize'},       $fileNameLength,
-	  $extraFieldLength,                 $fileCommentLength,
-	  $self->{'diskNumberStart'},        $self->{'internalFileAttributes'},
-	  $self->{'externalFileAttributes'}, $self->{'localHeaderRelativeOffset'} )
+	(
+		$self->{'versionMadeBy'},          $self->{'fileAttributeFormat'},
+		$self->{'versionNeededToExtract'}, $self->{'bitFlag'},
+		$self->{'compressionMethod'},      $self->{'lastModFileDateTime'},
+		$self->{'crc32'},                  $self->{'compressedSize'},
+		$self->{'uncompressedSize'},       $fileNameLength,
+		$extraFieldLength,                 $fileCommentLength,
+		$self->{'diskNumberStart'},        $self->{'internalFileAttributes'},
+		$self->{'externalFileAttributes'}, $self->{'localHeaderRelativeOffset'}
+	)
 	  = unpack( CENTRAL_DIRECTORY_FILE_HEADER_FORMAT, $header );
 
 	$self->{'eocdCrc32'} = $self->{'crc32'};
@@ -2966,6 +2979,13 @@ sub _readCentralDirectoryFileHeader    # Archive::Zip::ZipFileMember
 		}
 	}
 
+	# NK 10/21/04: added to avoid problems with manipulated headers
+	if (    $self->{'uncompressedSize'} != $self->{'compressedSize'}
+		and $self->{'compressionMethod'} == COMPRESSION_STORED )
+	{
+		$self->{'uncompressedSize'} = $self->{'compressedSize'};
+	}
+
 	$self->desiredCompressionMethod( $self->compressionMethod() );
 
 	return AZ_OK;
