@@ -1,16 +1,21 @@
-# $Revision: 1.9 $
+# $Revision: 1.11 $
 #
+
 =head1 NAME
 
-Archive::Zip::Tree -- methods for adding/extracting trees using Archive::Zip
+Archive::Zip::Tree - methods for adding/extracting trees using Archive::Zip
 
 =head1 SYNOPSIS
 
+=head1 DESCRIPTION
+
 This module is deprecated, because all its methods were moved into the main
 Archive::Zip module.
 
 It is included in the distribution merely to avoid breaking old code.
 
+See L<Archive::Zip>.
+
 =head1 AUTHOR
 
 Ned Konz, perl@bike-nomad.com
@@ -27,6 +32,8 @@ L<Archive::Zip>
 
 =cut
 
+use Archive::Zip;
+
 warn("Archive::Zip::Tree is deprecated; its methods have been moved into Archive::Zip.") if $^W;
 
 1;
